@@ -31,6 +31,7 @@
 #include "parse-datetime.h"
 #include "ostree-repo-private.h"
 #include "ostree-libarchive-private.h"
+#include "ostree-sign.h"
 
 static char *opt_subject;
 static char *opt_body;
@@ -53,17 +54,22 @@ static gboolean opt_tar_autocreate_parents;
 static char *opt_tar_pathname_filter;
 static gboolean opt_no_xattrs;
 static char *opt_selinux_policy;
+static gboolean opt_selinux_policy_from_base;
 static gboolean opt_canonical_permissions;
+static gboolean opt_ro_executables;
 static gboolean opt_consume;
 static gboolean opt_devino_canonical;
+static char *opt_base;
 static char **opt_trees;
 static gint opt_owner_uid = -1;
 static gint opt_owner_gid = -1;
 static gboolean opt_table_output;
 #ifndef OSTREE_DISABLE_GPGME
-static char **opt_key_ids;
+static char **opt_gpg_key_ids;
 static char *opt_gpg_homedir;
 #endif
+static char **opt_key_ids;
+static char *opt_sign_name;
 static gboolean opt_generate_sizes;
 static gboolean opt_disable_fsync;
 static char *opt_timestamp;
@@ -88,7 +94,7 @@ parse_fsync_cb (const char  *option_name,
  */
 
 static GOptionEntry options[] = {
-  { "parent", 0, 0, G_OPTION_ARG_STRING, &opt_parent, "Parent ref, or \"none\"", "REF" },
+  { "parent", 0, 0, G_OPTION_ARG_STRING, &opt_parent, "Parent commit checksum, or \"none\"", "COMMIT" },
   { "subject", 's', 0, G_OPTION_ARG_STRING, &opt_subject, "One line subject", "SUBJECT" },
   { "body", 'm', 0, G_OPTION_ARG_STRING, &opt_body, "Full description", "BODY" },
   { "body-file", 'F', 0, G_OPTION_ARG_FILENAME, &opt_body_file, "Commit message from FILE path", "FILE" },
@@ -97,6 +103,7 @@ static GOptionEntry options[] = {
   { "orphan", 0, 0, G_OPTION_ARG_NONE, &opt_orphan, "Create a commit without writing a ref", NULL },
   { "no-bindings", 0, 0, G_OPTION_ARG_NONE, &opt_no_bindings, "Do not write any ref bindings", NULL },
   { "bind-ref", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_bind_refs, "Add a ref to ref binding commit metadata", "BRANCH" },
+  { "base", 0, 0, G_OPTION_ARG_STRING, &opt_base, "Start from the given commit as a base (no modifiers apply)", "REV" },
   { "tree", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_trees, "Overlay the given argument as a tree", "dir=PATH or tar=TARFILE or ref=COMMIT" },
   { "add-metadata-string", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_metadata_strings, "Add a key/value pair to metadata", "KEY=VALUE" },
   { "add-metadata", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_metadata_variants, "Add a key/value pair to metadata, where the KEY is a string, an VALUE is g_variant_parse() formatted", "KEY=VALUE" },
@@ -105,8 +112,10 @@ static GOptionEntry options[] = {
   { "owner-uid", 0, 0, G_OPTION_ARG_INT, &opt_owner_uid, "Set file ownership user id", "UID" },
   { "owner-gid", 0, 0, G_OPTION_ARG_INT, &opt_owner_gid, "Set file ownership group id", "GID" },
   { "canonical-permissions", 0, 0, G_OPTION_ARG_NONE, &opt_canonical_permissions, "Canonicalize permissions in the same way bare-user does for hardlinked files", NULL },
+  { "mode-ro-executables", 0, 0, G_OPTION_ARG_NONE, &opt_ro_executables, "Ensure executable files are not writable", NULL },
   { "no-xattrs", 0, 0, G_OPTION_ARG_NONE, &opt_no_xattrs, "Do not import extended attributes", NULL },
   { "selinux-policy", 0, 0, G_OPTION_ARG_FILENAME, &opt_selinux_policy, "Set SELinux labels based on policy in root filesystem PATH (may be /)", "PATH" },
+  { "selinux-policy-from-base", 'P', 0, G_OPTION_ARG_NONE, &opt_selinux_policy_from_base, "Set SELinux labels based on first --tree argument", NULL },
   { "link-checkout-speedup", 0, 0, G_OPTION_ARG_NONE, &opt_link_checkout_speedup, "Optimize for commits of trees composed of hardlinks into the repository", NULL },
   { "devino-canonical", 'I', 0, G_OPTION_ARG_NONE, &opt_devino_canonical, "Assume hardlinked objects are unmodified.  Implies --link-checkout-speedup", NULL },
   { "tar-autocreate-parents", 0, 0, G_OPTION_ARG_NONE, &opt_tar_autocreate_parents, "When loading tar archives, automatically create parent directories as needed", NULL },
@@ -117,9 +126,11 @@ static GOptionEntry options[] = {
   { "consume", 0, 0, G_OPTION_ARG_NONE, &opt_consume, "Consume (delete) content after commit (for local directories)", NULL },
   { "table-output", 0, 0, G_OPTION_ARG_NONE, &opt_table_output, "Output more information in a KEY: VALUE format", NULL },
 #ifndef OSTREE_DISABLE_GPGME
-  { "gpg-sign", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_key_ids, "GPG Key ID to sign the commit with", "KEY-ID"},
+  { "gpg-sign", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_gpg_key_ids, "GPG Key ID to sign the commit with", "KEY-ID"},
   { "gpg-homedir", 0, 0, G_OPTION_ARG_FILENAME, &opt_gpg_homedir, "GPG Homedir to use when looking for keyrings", "HOMEDIR"},
 #endif
+  { "sign", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_key_ids, "Sign the commit with", "KEY_ID"},
+  { "sign-type", 0, 0, G_OPTION_ARG_STRING, &opt_sign_name, "Signature type to use (defaults to 'ed25519')", "NAME"},
   { "generate-sizes", 0, 0, G_OPTION_ARG_NONE, &opt_generate_sizes, "Generate size information along with commit metadata", NULL },
   { "disable-fsync", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &opt_disable_fsync, "Do not invoke fsync()", NULL },
   { "fsync", 0, 0, G_OPTION_ARG_CALLBACK, parse_fsync_cb, "Specify how to invoke fsync()", "POLICY" },
@@ -185,13 +196,19 @@ commit_filter (OstreeRepo         *self,
     g_file_info_set_attribute_uint32 (file_info, "unix::uid", opt_owner_uid);
   if (opt_owner_gid >= 0)
     g_file_info_set_attribute_uint32 (file_info, "unix::gid", opt_owner_gid);
+  guint mode = g_file_info_get_attribute_uint32 (file_info, "unix::mode");
+
+  if (S_ISREG (mode) && opt_ro_executables && (mode & (S_IXUSR | S_IXGRP | S_IXOTH)))
+    {
+      mode &= ~(S_IWUSR | S_IWGRP | S_IWOTH);
+      g_file_info_set_attribute_uint32 (file_info, "unix::mode", mode);
+    }
 
   if (mode_adds && g_hash_table_lookup_extended (mode_adds, path, NULL, &value))
     {
-      guint current_mode = g_file_info_get_attribute_uint32 (file_info, "unix::mode");
       guint mode_add = GPOINTER_TO_UINT (value);
       g_file_info_set_attribute_uint32 (file_info, "unix::mode",
-                                        current_mode | mode_add);
+                                        mode | mode_add);
       g_hash_table_remove (mode_adds, path);
     }
   else if (mode_overrides && g_hash_table_lookup_extended (mode_overrides, path, NULL, &value))
@@ -417,6 +434,7 @@ ostree_builtin_commit (int argc, char **argv, OstreeCommandInvocation *invocatio
   OstreeRepoTransactionStats stats;
   struct CommitFilterData filter_data = { 0, };
   g_autofree char *commit_body = NULL;
+  g_autoptr (OstreeSign) sign = NULL;
 
   context = g_option_context_new ("[PATH]");
 
@@ -550,6 +568,11 @@ ostree_builtin_commit (int argc, char **argv, OstreeCommandInvocation *invocatio
     flags |= OSTREE_REPO_COMMIT_MODIFIER_FLAGS_GENERATE_SIZES;
   if (opt_disable_fsync)
     ostree_repo_set_disable_fsync (repo, TRUE);
+  if (opt_selinux_policy && opt_selinux_policy_from_base)
+    {
+      glnx_throw (error, "Cannot specify both --selinux-policy and --selinux-policy-from-base");
+      goto out;
+    }
 
   if (flags != 0
       || opt_owner_uid >= 0
@@ -557,25 +580,14 @@ ostree_builtin_commit (int argc, char **argv, OstreeCommandInvocation *invocatio
       || opt_statoverride_file != NULL
       || opt_skiplist_file != NULL
       || opt_no_xattrs
-      || opt_selinux_policy)
+      || opt_ro_executables
+      || opt_selinux_policy
+      || opt_selinux_policy_from_base)
     {
       filter_data.mode_adds = mode_adds;
       filter_data.skip_list = skip_list;
       modifier = ostree_repo_commit_modifier_new (flags, commit_filter,
                                                   &filter_data, NULL);
-      if (opt_selinux_policy)
-        {
-          glnx_autofd int rootfs_dfd = -1;
-          if (!glnx_opendirat (AT_FDCWD, opt_selinux_policy, TRUE, &rootfs_dfd, error))
-            {
-              g_prefix_error (error, "selinux-policy: ");
-              goto out;
-            }
-          policy = ostree_sepolicy_new_at (rootfs_dfd, cancellable, error);
-          if (!policy)
-            goto out;
-          ostree_repo_commit_modifier_set_sepolicy (modifier, policy);
-        }
     }
 
   if (opt_editor)
@@ -599,129 +611,180 @@ ostree_builtin_commit (int argc, char **argv, OstreeCommandInvocation *invocatio
   if (opt_link_checkout_speedup && !ostree_repo_scan_hardlinks (repo, cancellable, error))
     goto out;
 
-  mtree = ostree_mutable_tree_new ();
-
-  if (argc <= 1 && (opt_trees == NULL || opt_trees[0] == NULL))
+  if (opt_base)
     {
-      if (!ostree_repo_write_dfd_to_mtree (repo, AT_FDCWD, ".", mtree, modifier,
-                                           cancellable, error))
+      g_autofree char *base_commit = NULL;
+      g_autoptr(GFile) root = NULL;
+      if (!ostree_repo_read_commit (repo, opt_base, &root, &base_commit, cancellable, error))
         goto out;
-    }
-  else if (opt_trees != NULL)
-    {
-      const char *const*tree_iter;
-      const char *tree;
-      const char *eq;
+      OstreeRepoFile *rootf = (OstreeRepoFile*) root;
 
-      for (tree_iter = (const char *const*)opt_trees; *tree_iter; tree_iter++)
+      mtree = ostree_mutable_tree_new_from_checksum (repo,
+                                                     ostree_repo_file_tree_get_contents_checksum (rootf),
+                                                     ostree_repo_file_tree_get_metadata_checksum (rootf));
+
+      if (opt_selinux_policy_from_base)
         {
-          tree = *tree_iter;
-
-          eq = strchr (tree, '=');
-          if (!eq)
-            {
-              g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           "Missing type in tree specification '%s'", tree);
-              goto out;
-            }
-          g_free (tree_type);
-          tree_type = g_strndup (tree, eq - tree);
-          tree = eq + 1;
-
-          g_clear_object (&object_to_commit);
-          if (strcmp (tree_type, "dir") == 0)
-            {
-              if (!ostree_repo_write_dfd_to_mtree (repo, AT_FDCWD, tree, mtree, modifier,
-                                                   cancellable, error))
-                goto out;
-            }
-          else if (strcmp (tree_type, "tar") == 0)
-            {
-              if (!opt_tar_pathname_filter)
-                {
-                  if (strcmp (tree, "-") == 0)
-                    {
-                      if (!ostree_repo_write_archive_to_mtree_from_fd (repo, STDIN_FILENO, mtree, modifier,
-                                                                       opt_tar_autocreate_parents,
-                                                                       cancellable, error))
-                        goto out;
-                    }
-                  else
-                    {
-                      object_to_commit = g_file_new_for_path (tree);
-
-                      if (!ostree_repo_write_archive_to_mtree (repo, object_to_commit, mtree, modifier,
-                                                               opt_tar_autocreate_parents,
-                                                               cancellable, error))
-                        goto out;
-                    }
-                }
-              else
-                {
-#ifdef HAVE_LIBARCHIVE
-                  const char *comma = strchr (opt_tar_pathname_filter, ',');
-                  if (!comma)
-                    {
-                      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                                           "Missing ',' in --tar-pathname-filter");
-                      goto out;
-                    }
-                  const char *replacement = comma + 1;
-                  g_autofree char *regexp_text = g_strndup (opt_tar_pathname_filter, comma - opt_tar_pathname_filter);
-                  /* Use new API if we have a pathname filter */
-                  OstreeRepoImportArchiveOptions opts = { 0, };
-                  opts.autocreate_parents = opt_tar_autocreate_parents;
-                  opts.translate_pathname = handle_translate_pathname;
-                  g_autoptr(GRegex) regexp = g_regex_new (regexp_text, 0, 0, error);
-                  TranslatePathnameData tpdata = { regexp, replacement };
-                  if (!regexp)
-                    {
-                      g_prefix_error (error, "--tar-pathname-filter: ");
-                      goto out;
-                    }
-                  opts.translate_pathname_user_data = &tpdata;
-
-                  g_autoptr(OtAutoArchiveRead) archive;
-                  if (strcmp (tree, "-") == 0)
-                    archive = ot_open_archive_read_fd (STDIN_FILENO, error);
-                  else
-                    archive = ot_open_archive_read (tree, error);
-
-                  if (!archive)
-                    goto out;
-                  if (!ostree_repo_import_archive_to_mtree (repo, &opts, archive, mtree,
-                                                            modifier, cancellable, error))
-                    goto out;
-#else
-                  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
-                               "This version of ostree is not compiled with libarchive support");
-                  goto out;
-#endif
-                }
-            }
-          else if (strcmp (tree_type, "ref") == 0)
-            {
-              if (!ostree_repo_read_commit (repo, tree, &object_to_commit, NULL, cancellable, error))
-                goto out;
-
-              if (!ostree_repo_write_directory_to_mtree (repo, object_to_commit, mtree, modifier,
-                                                         cancellable, error))
-                goto out;
-            }
-          else
-            {
-              g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           "Invalid tree type specification '%s'", tree_type);
-              goto out;
-            }
+          g_assert (modifier);
+          if (!ostree_repo_commit_modifier_set_sepolicy_from_commit (modifier, repo, base_commit, cancellable, error))
+            goto out;
+          /* Don't try to handle it twice */
+          opt_selinux_policy_from_base = FALSE;
         }
     }
   else
     {
-      g_assert (argc > 1);
-      if (!ostree_repo_write_dfd_to_mtree (repo, AT_FDCWD, argv[1], mtree, modifier,
-                                           cancellable, error))
-        goto out;
+      mtree = ostree_mutable_tree_new ();
+    }
+
+
+  /* Convert implicit . or explicit path via argv into
+   * --tree=dir= so that we only have one primary code path below.
+   */
+  if (opt_trees == NULL || opt_trees[0] == NULL)
+    {
+      char *path;
+      if (argc <= 1)
+        path = ".";
+      else
+        path = argv[1];
+      opt_trees = g_new0 (char *, 2);
+      opt_trees[0] = g_strconcat ("dir=", path, NULL);
+    }
+
+  const char *const*tree_iter;
+  const char *tree;
+  const char *eq;
+  g_assert (opt_trees && *opt_trees);
+  for (tree_iter = (const char *const*)opt_trees; *tree_iter; tree_iter++)
+    {
+      const gboolean first = (tree_iter == (const char *const*)opt_trees);
+      tree = *tree_iter;
+
+      eq = strchr (tree, '=');
+      if (!eq)
+        {
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                        "Missing type in tree specification '%s'", tree);
+          goto out;
+        }
+      g_free (tree_type);
+      tree_type = g_strndup (tree, eq - tree);
+      tree = eq + 1;
+
+      g_clear_object (&object_to_commit);
+      if (strcmp (tree_type, "dir") == 0)
+        {
+          if (first && opt_selinux_policy_from_base)
+            {
+              opt_selinux_policy = g_strdup (tree);
+              opt_selinux_policy_from_base = FALSE;
+            }
+          if (first && opt_selinux_policy)
+            {
+              g_assert (modifier);
+              glnx_autofd int rootfs_dfd = -1;
+              if (!glnx_opendirat (AT_FDCWD, opt_selinux_policy, TRUE, &rootfs_dfd, error))
+                goto out;
+              policy = ostree_sepolicy_new_at (rootfs_dfd, cancellable, error);
+              if (!policy)
+                goto out;
+              ostree_repo_commit_modifier_set_sepolicy (modifier, policy);
+            }
+          if (!ostree_repo_write_dfd_to_mtree (repo, AT_FDCWD, tree, mtree, modifier,
+                                                cancellable, error))
+            goto out;
+        }
+      else if (strcmp (tree_type, "tar") == 0)
+        {
+          if (first && opt_selinux_policy_from_base)
+            {
+              glnx_throw (error, "Cannot use --selinux-policy-from-base with tar");
+              goto out;
+            }
+          if (!opt_tar_pathname_filter)
+            {
+              if (strcmp (tree, "-") == 0)
+                {
+                  if (!ostree_repo_write_archive_to_mtree_from_fd (repo, STDIN_FILENO, mtree, modifier,
+                                                                    opt_tar_autocreate_parents,
+                                                                    cancellable, error))
+                    goto out;
+                }
+              else
+                {
+                  object_to_commit = g_file_new_for_path (tree);
+
+                  if (!ostree_repo_write_archive_to_mtree (repo, object_to_commit, mtree, modifier,
+                                                            opt_tar_autocreate_parents,
+                                                            cancellable, error))
+                    goto out;
+                }
+            }
+          else
+            {
+#ifdef HAVE_LIBARCHIVE
+              const char *comma = strchr (opt_tar_pathname_filter, ',');
+              if (!comma)
+                {
+                  g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                                        "Missing ',' in --tar-pathname-filter");
+                  goto out;
+                }
+              const char *replacement = comma + 1;
+              g_autofree char *regexp_text = g_strndup (opt_tar_pathname_filter, comma - opt_tar_pathname_filter);
+              /* Use new API if we have a pathname filter */
+              OstreeRepoImportArchiveOptions opts = { 0, };
+              opts.autocreate_parents = opt_tar_autocreate_parents;
+              opts.translate_pathname = handle_translate_pathname;
+              g_autoptr(GRegex) regexp = g_regex_new (regexp_text, 0, 0, error);
+              TranslatePathnameData tpdata = { regexp, replacement };
+              if (!regexp)
+                {
+                  g_prefix_error (error, "--tar-pathname-filter: ");
+                  goto out;
+                }
+              opts.translate_pathname_user_data = &tpdata;
+
+              g_autoptr(OtAutoArchiveRead) archive;
+              if (strcmp (tree, "-") == 0)
+                archive = ot_open_archive_read_fd (STDIN_FILENO, error);
+              else
+                archive = ot_open_archive_read (tree, error);
+
+              if (!archive)
+                goto out;
+              if (!ostree_repo_import_archive_to_mtree (repo, &opts, archive, mtree,
+                                                        modifier, cancellable, error))
+                goto out;
+#else
+              g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+                            "This version of ostree is not compiled with libarchive support");
+              goto out;
+#endif
+            }
+        }
+      else if (strcmp (tree_type, "ref") == 0)
+        {
+          if (first && opt_selinux_policy_from_base)
+            {
+              g_assert (modifier);
+              if (!ostree_repo_commit_modifier_set_sepolicy_from_commit (modifier, repo, tree, cancellable, error))
+                goto out;
+            }
+          if (!ostree_repo_read_commit (repo, tree, &object_to_commit, NULL, cancellable, error))
+            goto out;
+
+          if (!ostree_repo_write_directory_to_mtree (repo, object_to_commit, mtree, modifier,
+                                                      cancellable, error))
+            goto out;
+        }
+      else
+        {
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                        "Invalid tree type specification '%s'", tree_type);
+          goto out;
+        }
     }
 
   if (mode_adds && g_hash_table_size (mode_adds) > 0)
@@ -811,12 +874,41 @@ ostree_builtin_commit (int argc, char **argv, OstreeCommandInvocation *invocatio
             goto out;
         }
 
-#ifndef OSTREE_DISABLE_GPGME
       if (opt_key_ids)
         {
+          /* Initialize crypto system */
+          opt_sign_name = opt_sign_name ?: OSTREE_SIGN_NAME_ED25519;
+
+          sign = ostree_sign_get_by_name (opt_sign_name, error);
+          if (sign == NULL)
+            goto out;
+
           char **iter;
 
           for (iter = opt_key_ids; iter && *iter; iter++)
+            {
+              const char *keyid = *iter;
+              g_autoptr (GVariant) secret_key = NULL;
+
+              secret_key = g_variant_new_string (keyid);
+              if (!ostree_sign_set_sk (sign, secret_key, error))
+                  goto out;
+
+              if (!ostree_sign_commit (sign,
+                                       repo,
+                                       commit_checksum,
+                                       cancellable,
+                                       error))
+                goto out;
+            }
+        }
+
+#ifndef OSTREE_DISABLE_GPGME
+      if (opt_gpg_key_ids)
+        {
+          char **iter;
+
+          for (iter = opt_gpg_key_ids; iter && *iter; iter++)
             {
               const char *keyid = *iter;
 
